package main

import (
	"bufio"
	"bytes"
	"encoding/binary"
	"fmt"
	"os"
	"runtime"
	"unicode/utf16"
	"unicode/utf8"
	_ "github.com/mattn/go-sqlite3"
	"database/sql"
	"strings"
	"log"
	"strconv"
)

// UTF16BytesToString converts UTF-16 encoded bytes, in big or little endian byte order,
// to a UTF-8 encoded string.
func UTF16BytesToString(b []byte, o binary.ByteOrder) string {
	utf := make([]uint16, (len(b)+(2-1))/2)
	for i := 0; i+(2-1) < len(b); i += 2 {
		utf[i/2] = o.Uint16(b[i:])
	}
	if len(b)/2 < len(utf) {
		utf[len(utf)-1] = utf8.RuneError
	}
	return string(utf16.Decode(utf))
}

// UTF-16 endian byte order
const (
	unknownEndian = iota
	bigEndian
	littleEndian
)

// dropCREndian drops a terminal \r from the endian data.
func dropCREndian(data []byte, t1, t2 byte) []byte {
	if len(data) > 1 {
		if data[len(data)-2] == t1 && data[len(data)-1] == t2 {
			return data[0 : len(data)-2]
		}
	}
	return data
}

// dropCRBE drops a terminal \r from the big endian data.
func dropCRBE(data []byte) []byte {
	return dropCREndian(data, '\x00', '\r')
}

// dropCRLE drops a terminal \r from the little endian data.
func dropCRLE(data []byte) []byte {
	return dropCREndian(data, '\r', '\x00')
}

// dropCR drops a terminal \r from the data.
func dropCR(data []byte) ([]byte, int) {
	var endian = unknownEndian
	switch ld := len(data); {
	case ld != len(dropCRLE(data)):
		endian = littleEndian
	case ld != len(dropCRBE(data)):
		endian = bigEndian
	}
	return data, endian
}

// SplitFunc is a split function for a Scanner that returns each line of
// text, stripped of any trailing end-of-line marker. The returned line may
// be empty. The end-of-line marker is one optional carriage return followed
// by one mandatory newline. In regular expression notation, it is `\r?\n`.
// The last non-empty line of input will be returned even if it has no
// newline.
func ScanUTF16LinesFunc(byteOrder binary.ByteOrder) (bufio.SplitFunc, func() binary.ByteOrder) {

	// Function closure variables
	var endian = unknownEndian
	switch byteOrder {
	case binary.BigEndian:
		endian = bigEndian
	case binary.LittleEndian:
		endian = littleEndian
	}
	const bom = 0xFEFF
	var checkBOM bool = endian == unknownEndian

	// Scanner split function
	splitFunc := func(data []byte, atEOF bool) (advance int, token []byte, err error) {

		if atEOF && len(data) == 0 {
			return 0, nil, nil
		}

		if checkBOM {
			checkBOM = false
			if len(data) > 1 {
				switch uint16(bom) {
				case uint16(data[0])<<8 | uint16(data[1]):
					endian = bigEndian
					return 2, nil, nil
				case uint16(data[1])<<8 | uint16(data[0]):
					endian = littleEndian
					return 2, nil, nil
				}
			}
		}

		// Scan for newline-terminated lines.
		i := 0
		for {
			j := bytes.IndexByte(data[i:], '\n')
			if j < 0 {
				break
			}
			i += j
			switch e := i % 2; e {
			case 1: // UTF-16BE
				if endian != littleEndian {
					if i > 1 {
						if data[i-1] == '\x00' {
							endian = bigEndian
							// We have a full newline-terminated line.
							return i + 1, dropCRBE(data[0 : i-1]), nil
						}
					}
				}
			case 0: // UTF-16LE
				if endian != bigEndian {
					if i+1 < len(data) {
						i++
						if data[i] == '\x00' {
							endian = littleEndian
							// We have a full newline-terminated line.
							return i + 1, dropCRLE(data[0 : i-1]), nil
						}
					}
				}
			}
			i++
		}

		// If we're at EOF, we have a final, non-terminated line. Return it.
		if atEOF {
			// drop CR.
			advance = len(data)
			switch endian {
			case bigEndian:
				data = dropCRBE(data)
			case littleEndian:
				data = dropCRLE(data)
			default:
				data, endian = dropCR(data)
			}
			if endian == unknownEndian {
				if runtime.GOOS == "windows" {
					endian = littleEndian
				} else {
					endian = bigEndian
				}
			}
			return advance, data, nil
		}

		// Request more data.
		return 0, nil, nil
	}

	// Endian byte order function
	orderFunc := func() (byteOrder binary.ByteOrder) {
		switch endian {
		case bigEndian:
			byteOrder = binary.BigEndian
		case littleEndian:
			byteOrder = binary.LittleEndian
		}
		return byteOrder
	}

	return splitFunc, orderFunc
}

func main() {
	//----------------------------------------
	//SETUP VARIABLES-------------------------
		counter := 0

	//----------------------------------------
	//SQLITE DB REFERENCE---------------------
		db, err := sql.Open("sqlite3", "/home/frick/go/src/bitbucket.org/prmunty/condensermain/prime_unity.db")
		if err != nil {
			log.Fatal(err)
		}

		_, err2 := db.Exec("DELETE FROM languages")
		if err2 != nil {
			log.Fatal(err)
		}



	defer db.Close()
	//----------------------------------------
	//SETUP UNICODE.TXT FILE------------------
		file, err := os.Open("unicode.txt")
		if err != nil {
			fmt.Println(err)
			os.Exit(1)
		}
		defer file.Close()
		fmt.Println(file.Name())


		rdr := bufio.NewReader(file)
		scanner := bufio.NewScanner(rdr)
		var bo binary.ByteOrder // unknown, infer from data
		// bo = binary.LittleEndian // windows
		splitFunc, orderFunc := ScanUTF16LinesFunc(bo)
		scanner.Split(splitFunc)

	//----------------------------------------
	//START LOOPING THROUGH FILE--------------
		for scanner.Scan() {
			b := scanner.Bytes()
			s := UTF16BytesToString(b, orderFunc())

			if (counter % 4 == 0){ //ENGLISH
				tx, err := db.Begin()
				if err != nil {
					log.Fatal(err)
				}
				stmt, err := tx.Prepare("insert into languages(language_id, name, english) values(?, ?, ?)")
				defer stmt.Close()
				_, err = stmt.Exec(strconv.Itoa((counter/4)-1), strings.ToLower(strings.Replace(strings.Replace(strings.Replace(strings.Replace(s, " ", "_", -1), "-", "_", -1), ".", "", -1), "/", "_", -1)), s)
				if err != nil {
					log.Fatal(err)
				}

				tx.Commit()

				fmt.Println("language:" + strings.ToLower(strings.Replace(strings.Replace(s, " ", "_", -1), "-", "_", -1)) + ":ENGLISH")

			}else if (counter % 4 == 1){ //FRENCH
				tx, err := db.Begin()
				if err != nil {
					log.Fatal(err)
				}
				stmt, err := tx.Prepare("UPDATE languages SET french=? WHERE language_id = ?")
				defer stmt.Close()
				_, err = stmt.Exec(s, strconv.Itoa((counter/4)-1))
				if err != nil {
					log.Fatal(err)
				}

				tx.Commit()

				fmt.Println("UPDATE languages SET french=" +  strings.Replace(s, "'", "''", -1) + "WHERE language_id = " + strconv.Itoa(counter-1))

			}else if (counter % 4 == 2){ //Portuguese
				tx, err := db.Begin()
				if err != nil {
					log.Fatal(err)
				}
				stmt, err := tx.Prepare("UPDATE languages SET portuguese=? WHERE language_id = ?")
				defer stmt.Close()
				_, err = stmt.Exec(s, strconv.Itoa((counter/4)-1))
				if err != nil {
					log.Fatal(err)
				}

				tx.Commit()

				fmt.Println("UPDATE languages SET portuguese=" +  strings.Replace(s, "'", "''", -1) + "WHERE language_id = " + strconv.Itoa(counter-2))
			}else if (counter % 4 == 3){ //Spanish
				tx, err := db.Begin()
				if err != nil {
					log.Fatal(err)
				}
				stmt, err := tx.Prepare("UPDATE languages SET spanish=? WHERE language_id = ?")
				defer stmt.Close()
				_, err = stmt.Exec(s, strconv.Itoa((counter/4)-1))
				if err != nil {
					log.Fatal(err)
				}

				tx.Commit()

				fmt.Println("UPDATE languages SET portuguese=" +  strings.Replace(s, "'", "''", -1) + "WHERE language_id = " + strconv.Itoa(counter-3))
			}

			counter += 1 //INCREMENT COUNTER
		}
	//fmt.Println(orderFunc())

	if err := scanner.Err(); err != nil {
		fmt.Println(err)
	}
}
